require_relative "version"

module Condor
  
  class Job
    
    attr_reader :priority, :id, :cmd, :dir, :job_data
    attr_reader :finished

    def initialize(cmd = nil, options = {})
      options ||= {}
      @priority = options.delete(:priority)
      @avoid_finalize = !!options.delete(:no_finalizer)
      @home = `echo $HOME`.strip
      @home = nil if (@home || '').empty?
      @dir = options.delete(:directory) || Dir.pwd
      @id = options.delete(:id)
      @finished = false
      @job_data = nil
      @base_name = options.delete(:base_name) || "#{@dir}/cinput"
      @suppress_numbering = options.delete(:simple)
      
      options.empty? or raise "Unknown options: #{options.keys.inspect}"

      if @id
        @cmd = nil
        update
      else
        @id = nil
        @cmd = []
        @cmd << "#!/bin/bash"
        @cmd << "# encoding: UTF-8"
        @cmd << "source /etc/profile"
        @cmd << "source #{@home}/.profile" if @home and File.exist?("#{@home}/.profile")
        @cmd << "source #{@home}/.rvm/scripts/rvm" if @home and File.exist?("#{@home}/.rvm/scripts/rvm")        
        @cmd += [cmd].flatten
      end
      
      ObjectSpace.define_finalizer(self, proc { finalize_job }) unless @avoid_finalize
    end

    def run(set_dir = nil)
      @dir = set_dir || @dir
      name = if @suppress_numbering
        "#{@base_name}"
      else
        # get condor input file number
        num = (Dir["#{@base_name}*"].map{|x| File.basename(x).sub(@base_name,'').to_i}.max or 0) + 1
        "#{@base_name}#{num}"
      end
      
      @input, @log, @error, @output, @bash = ['.inp','.log','.error','.output',''].map{|x| "#{name}#{x}"}

      # create bash executable file to run
      File.open(@bash,'w'){|f| f.puts @cmd * "\n"}
      `chmod +x #{@bash}`

      input = []
      input << "Executable = #{@bash}"
      input << "Universe = vanilla"
      input << "Log = #{name}.log"
      input << "Error = #{name}.error"
      input << "Environment = HOME=#{@home}" if @home
      input << "Initialdir = #{@dir}"
      input << "Output = #{@output}"
      input << "Requirements = TARGET.OpSys == \"LINUX\""
      input << "Queue"

      File.open(@input, 'w'){|f| f.puts input * "\n"}
      if submit_output = `condor_submit #{@input}`.strip.split.last
        @id = submit_output + '0'
        set_priority
        update(true)
        @executed = true
      else
        @id = nil
        @executed = false
        @job_data = nil
      end        
    end

    def cleanup
      [@input, @log, @error, @output, @bash].each{|x| `rm #{x}` if x and File.exists?(x)} if @id
    end

    def priority=(val)
      @priority = val
      set_priority
    end

    def done?; @finished; end
    def running?; @job_data ? (@job_data.job_status == '2') : false; end
    def idle?; @job_data ? (@job_data.job_status == '1') : false; end
    def held?; @job_data ? (@job_data.job_status == '5') : false; end
    def runtime; @job_data ? Time.at(@job_data.server_time.to_f - @job_data.job_start_date.to_f).utc.strftime("%H:%M:%S") : nil; end
    def alive?; @id and !@finished; end

    def error; IO.read(@error); end
    def input; IO.read(@input); end
    def log; IO.read(@log); end
    def output; IO.read(@output); end
    def bash; IO.read(@bash); end

    # update job information
    def update(force = false)
      Condor::Queue.load(force)
      return nil if !@id
      if @job_data = Condor::Queue[@id]
        return self
      else
        @finished = true 
        return nil
      end
    end
    
    def remove
      `condor_rm #{@id}` if @id
    end

    def finalize_job
      unless @avoid_finalize
        remove if !done?
        cleanup
      end
    end

  private

    def set_priority
      `condor_prio -p #{@priority} #{@id}` if alive?
    end
    
  end
end
